@@ -100,7 +100,7 @@ void mmd_print_char_latex(DString * out, char c) {
 			break;
 
 		case '/':
-			print_const("\\slash ");
+			print_const("\\slash{}");
 			break;
 
 		case '^':
@@ -271,16 +271,20 @@ void mmd_export_link_latex(DString * out, const char * source, token * text, lin
 
 				if (temp_char && temp_char[0] != '\0') {
 					mmd_export_token_tree_latex(out, source, text->child, scratch);
-					print_const(" (");
-					printf("\\autoref{%s}", &(link->url)[1]);
-					print_const(")");
+					print_const(" (\\autoref{");
+					mmd_print_string_latex(out, &(link->url)[1]);
+					print_const("})");
 				} else {
-					printf("\\autoref{%s}", &(link->url)[1]);
+					print_const("\\autoref{");
+					mmd_print_string_latex(out, &(link->url)[1]);
+					print_const("}");
 				}
 
 				free(temp_char);
 			} else {
-				printf("\\autoref{%s}", &(link->url)[1]);
+				print_const("\\autoref{");
+				mmd_print_string_latex(out, &(link->url)[1]);
+				print_const("}");
 			}
 
 			return;
@@ -411,7 +415,7 @@ void mmd_export_image_latex(DString * out, const char * source, token * text, li
 	if (is_figure) {
 		print_const("\n");
 
-		if (text) {
+		if ((text && text->len > 3) || (link->title && link->title[0] != '\0')) {
 			if (link->title && link->title[0] != '\0') {
 				printf("\\caption[%s]{", link->title);
 			} else {
@@ -485,9 +489,11 @@ void mmd_export_toc_entry_latex(DString * out, const char * source, scratch_pad
 void mmd_export_toc_latex(DString * out, const char * source, scratch_pad * scratch) {
 	size_t counter = 0;
 
+	int old_label_counter = scratch->label_counter;
+
 	mmd_export_toc_entry_latex(out, source, scratch, &counter, 0);
 
-	scratch->label_counter = 0;
+	scratch->label_counter = old_label_counter;
 }
 
 
@@ -554,6 +560,7 @@ void mmd_export_token_latex(DString * out, const char * source, token * t, scrat
 					// Raw source
 					if (raw_filter_text_matches(temp_char, FORMAT_LATEX)) {
 						switch (t->child->tail->type) {
+							case CODE_FENCE_LINE:
 							case LINE_FENCE_BACKTICK_3:
 							case LINE_FENCE_BACKTICK_4:
 							case LINE_FENCE_BACKTICK_5:
@@ -642,6 +649,9 @@ void mmd_export_token_latex(DString * out, const char * source, token * t, scrat
 			scratch->padded = 1;
 			break;
 
+		case MARKER_DEFLIST_COLON:
+			break;
+
 		case BLOCK_EMPTY:
 			break;
 
@@ -698,6 +708,7 @@ void mmd_export_token_latex(DString * out, const char * source, token * t, scrat
 					break;
 			}
 
+			header_clean_trailing_whitespace(t->child, source);
 			mmd_export_token_tree_latex(out, source, t->child, scratch);
 			trim_trailing_whitespace_d_string(out);
 
@@ -1133,6 +1144,8 @@ void mmd_export_token_latex(DString * out, const char * source, token * t, scrat
 		case MARKER_H4:
 		case MARKER_H5:
 		case MARKER_H6:
+		case MARKER_SETEXT_1:
+		case MARKER_SETEXT_2:
 		case MARKER_LIST_BULLET:
 		case MARKER_LIST_ENUMERATOR:
 			break;
@@ -1843,7 +1856,7 @@ parse_citation:
 			break;
 
 		case SLASH:
-			print_const("\\slash ");
+			print_const("\\slash{}");
 			break;
 
 		case STAR:
@@ -2002,6 +2015,9 @@ parse_citation:
 			print_const("\\_");
 			break;
 
+		case OBJECT_REPLACEMENT_CHARACTER:
+			break;
+
 		default:
 			fprintf(stderr, "Unknown token type: %d\n", t->type);
 			token_describe(t, source);
@@ -2272,7 +2288,7 @@ void mmd_export_token_latex_tt(DString * out, const char * source, token * t, sc
 			break;
 
 		case SLASH:
-			print_const("\\slash ");
+			print_const("\\slash{}");
 			break;
 
 		case TEXT_BACKSLASH:
@@ -2461,7 +2477,7 @@ void mmd_start_complete_latex(DString * out, const char * source, scratch_pad *
 			print_const("}\n");
 		} else if (strcmp(m->key, "bibtex") == 0) {
 			print_const("\\def\\bibliocommand{\\bibliography{");
-			mmd_print_string_latex(out, m->value);
+			print(m->value);
 			print_const("}}\n");
 		} else if (strcmp(m->key, "transcludebase") == 0) {
 		} else if (strcmp(m->key, "xhtmlheader") == 0) {
